define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/dom-style",
    "thornberry/app/component/Report/View",
    "thornberry/app/utils",
    "thornberry/app/Service/Settings",
    "thornberry/app/Service/NoeNotr",
    "thornberry/app/Service/EnsembleFactory",
    "dijit/Menu", 
    "dijit/MenuItem", 
    "dijit/form/ComboButton"
],function(
    declare,
    lang,
    arrayUtils,
    domStyle,
    ReportView,
    utils,
    settingsService,
    service,
    ensembleFactory,
    Menu,
    MenuItem,
    ComboButton
) {
    return declare(ReportView,{

        settings: null,

        getInitialProps: function() {
            return lang.mixin(this.inherited(arguments),{
                type: "NOE" // Or "NOTR"
            });
        },

        getPageProps: function() {
            var t = this;
            var selectedIds = t.props.selectedIds;
            var cannotGenerate = selectedIds.length === 0 || arrayUtils.some(selectedIds,function(id) {
                var status = t.getGridData(id)[lang.replace("{type}Status",t.props)];
                return status === "Accepted" || status === "Pending";
            });
            var cannotEdit = selectedIds.length !== 1 || t.getGridData(selectedIds[0])[lang.replace("{type}Status",t.props)] === "Pending";
            var cannotProcess = selectedIds.length === 0 || arrayUtils.some(selectedIds,function(id) {
                var status = t.getGridData(id)[lang.replace("{type}Status",t.props)];
                return status !== "Response Received";
            });
            var canPrint = t.props && t.props.type == "NOE" && selectedIds.length == 1 && t.getGridData(selectedIds[0]).Printable == "Yes";

            return lang.mixin(t.inherited(arguments),{
                actions: [
                    { label: "Refresh",            onClick: lang.hitch(t,"refreshData") },
                    { label: "Upload Response",     onClick: function() { t.loadSubView("uploadFileView"); } },

                    [
                        { label: "Edit",                                    onClick: lang.hitch(t,"_edit"),     hidden: cannotEdit },
                        { label: "Process",                                 onClick: lang.hitch(t,"process"),   hidden: cannotProcess },
                        { label: "History",                                 onClick: lang.hitch(t,"history"),   hidden: selectedIds.length !== 1 },
                        ( cannotGenerate ? null : t.settings.interfaceEnabled ? t._requestComboButton : { label: lang.replace("Generate {type} File",t.props),         onClick: function() { t.createRequest() } } ),
                        { label: "Print",                                   onClick: lang.hitch(t,"print"),     hidden: !canPrint },
                    ]
                ]
            });
        },

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);

            var menu = new Menu({ style: "display: none;"});
            var menuItem = new MenuItem({
                label: "Create File",
                onClick: function() { t.createRequest() }
            });
            menu.addChild(menuItem);

            t._requestComboButton = new ComboButton({
                label: lang.replace("Generate {type}",t.props),
                dropDown: menu,
                onClick: function() { t.createRequest(true) }
            });
            t._requestComboButton.startup();
        },

        refreshRendering: function() {
            var t = this;
            t.title = lang.replace("{type} Tracking",t.props);
            t.props.reportClass = lang.replace("Billing.Report.{type}Tracking",t.props);

            if (!t.settings) {
                t.addReadyPromise(settingsService.get("noeNotrTracking").then(function(settings) {
                    t.settings = settings;
                }));
            }

            t.inherited(arguments);
        },

        _edit: function() {
            this.edit(this.props.selectedIds[0]);
        },

        process: function() {
            var t = this;
            t.loadSubView("processResponse",{
                policyIds: t.props.selectedIds
            });
        },

        print: function() {
            var t = this;
            t.props.onSetLoadingPromise(service.printNoe(t.props.selectedIds[0]).then(function(fileLinkId) {
                utils.promptDownload(fileLinkId);
            },utils.showMessage));
        },

        onRowDoubleClick: function(id) {
            this.edit(id);
        },

        createRequest: function(useInterface) {
            var t = this;
            var service = ensembleFactory("Billing.NOE.EnsembleRequest");
            var promise = service({
                type: t.props.type,
                policyIds:t.props.selectedIds,
                useInterface:!!useInterface
            });
            promise.then(function(result) {
                t.loadSubView("requestSummary",result);
                t.refreshData();
            },utils.showMessage);
            t.props.onSetLoadingPromise(promise);
        },

        edit: function(id) {
            var t = this;
            t.loadSubView("editView",{
                objectId: id
            });
        },

        onShow: function() {
            var t = this;
            t.inherited(arguments);

            if (t.firstShow) {
                t.addSubView("editView","thornberry/app/Component/NoeNotrTracking/EditView",function() {
                    return { 
                        type: t.props.type,
                        onSave: lang.hitch(t,"refreshData")
                    };
                },true);
                t.addSubView("historyView","thornberry/app/Component/NoeNotrTracking/HistoryView",function() {
                    return { 
                        type: t.props.type
                    };
                },true);
                t.addSubView("requestSummary","thornberry/app/Component/NoeNotrTracking/RequestSummaryView",function() {
                    return {
                        type: t.props.type
                    };
                });
                t.addSubView("uploadFileView","thornberry/app/Component/FileUploadView",function() {
                    return { 
                        title: "Upload NOE Acceptance File",
                        serverClass: "Billing.NOE.Service",
                        serverMethod: lang.replace("Upload{type}File",t.props),
                        onUpload: function(promise) {
                            t.getSubView("uploadFileView").props.onCloseView();
                            var service = ensembleFactory(lang.replace("Billing.NOE.{type}EnsembleResponse",t.props));
                            var promise = service(promise)
                            promise.then(function(policyIds) {
                                t.loadSubView("processResponse",{
                                    policyIds: policyIds
                                });
                                t.refreshData();
                            },utils.showMessage);
                            t.props.onSetLoadingPromise(promise,"Processing...");
                        }
                    };
                });
                t.addSubView("processResponse","thornberry/app/Component/NoeNotrTracking/ProcessResponseView",function() {
                    return {
                        type: t.props.type,
                        onProcess: function() {
                            t.refreshData();
                        }
                    };
                },true);
            }
        },

        onLoadController: function() {
            var t = this;
            var promise = t.inherited(arguments);
            domStyle.set(t.presetControlContainer,"text-align","center");
            domStyle.set(t.presetControlContainer,"width","300px");
            domStyle.set(t.presetControlContainer,"position","absolute");
            domStyle.set(t.presetControlContainer,"right","200px");
            return promise;
        },

        history: function() {
            var t = this;
            t.loadSubView("historyView",{
                summary: t.getGridData(t.props.selectedIds[0]).DetailSummary,
                type: t.props.type,
                policyId: t.props.selectedIds[0]
            });
        }
    });
});