define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "thornberry/serverRequest",
    "dojo/Deferred"
],function(
    declare,
    lang,
    serverRequest,
    Deferred
) {
    var DASHBOARD_SERVER_CLASS = "Billing.Claim.DashboardService";
    var SERVER_CLASS = "Billing.Claim.Service";

    var Service = declare(null,{
        getDashboardData: function() {
            return serverRequest(DASHBOARD_SERVER_CLASS,"GetData");
        },
        getClaim: function(id) {
            return serverRequest(SERVER_CLASS,"GetClaim",id);
        },
        printClaim: function(id) {
            return serverRequest(SERVER_CLASS,"PrintSavedX12",id);
        },
        findClaimId: function(claimId) {
            return serverRequest(SERVER_CLASS,"FindClaimId",claimId);
        },
        recalculateClaims: function(claimIds) {
            return serverRequest(SERVER_CLASS,"RecalculateClaims",claimIds,true);
        },
        toggleReadyToBill: function(claimIds,readyToBill) {
            return serverRequest(SERVER_CLASS,"ToggleReadyToBillStatus",{
                claimIds: claimIds,
                readyToBill: readyToBill
            });
        },
        submitClaims: function(claimIds,agingDate,prepayIds) {
            return serverRequest(SERVER_CLASS,"SubmitClaims",{
                claimIds: claimIds,
                prepayIds: prepayIds,
                agingDate: agingDate
            },true);
        },
        codeClaim: function(data) {
            return serverRequest(SERVER_CLASS,"CodeClaim",data);
        },
        voidClaims: function(claimIds) {
            return serverRequest(SERVER_CLASS,"VoidClaims",claimIds);
        },
        updateBilledStatus: function(claimIds,status) {
            return serverRequest(SERVER_CLASS,"UpdateBilledStatus",{
                claimIds: claimIds,
                status: status
            });
        },
        getClaimFileDownloadLink: function(claimFileId) {
            return serverRequest(SERVER_CLASS,"GetClaimFileDownloadLink",claimFileId);
        },
        getClaimFileDetails: function(claimFileId) {
            return serverRequest(SERVER_CLASS,"GetClaimFileDetails",claimFileId);
        },
        updateClaimFileStatus: function(claimFileIds,status) {
            return serverRequest(SERVER_CLASS,"UpdateClaimFileStatus",{
                claimFileIds: claimFileIds,
                status: status
            });
        },

        getClaimResponseFile: function(id) {
            return serverRequest(SERVER_CLASS,"GetClaimResponseFile",id);
        },
        processClaimResponseFile: function(id) {
            return serverRequest(SERVER_CLASS,"ProcessClaimResponseFile",id);
        },
        deleteClaimResponseFile: function(id) {
            return serverRequest(SERVER_CLASS,"DeleteClaimResponseFile",id);
        },

        getFinancialReports: function() {
            return serverRequest("Billing.Utility","GetFinancialReports");
        },
        getFinancialPeriod: function(id) {
            return serverRequest(SERVER_CLASS,"GetFinancialPeriod",id);
        },
        getNextFinancialPeriod: function(organizationalUnitId) {
            return serverRequest(SERVER_CLASS,"GetNextFinancialPeriod",organizationalUnitId);
        },
        saveFinancialPeriod: function(data) {
            return serverRequest(SERVER_CLASS,"SaveFinancialPeriod",data,true);
        },
        generateReportFromConfig: function(configId,presentationType) {
            return serverRequest(SERVER_CLASS,"GenerateReportFromConfig",{ configId: configId, presentationType: presentationType });
        },
        getFinancialPeriodDownloadLink: function(financialPeriodId) {
            return serverRequest(SERVER_CLASS,"GetFinancialPeriodDownloadLink",financialPeriodId);
        }
    },true);
    return new Service(); // Singleton
});